import { BlockLocation, BlockPermutation, BlockProperties, BlockType, BoolBlockProperty, EffectType, Entity, EntityScaleComponent, IBlockProperty, IntBlockProperty, ItemStack, ItemType, MinecraftBlockTypes, MinecraftEffectTypes, MinecraftItemTypes, Player, PlayerInventoryComponentContainer, StringBlockProperty, world, } from "mojang-minecraft"
import { ActionFormData, ModalFormData } from "mojang-minecraft-ui"
declare interface ModalFormData {
    title(titleText: string): ModalFormData
    dropdown(label: string, options: string[], defaultValueIndex?: number): ModalFormData
}
let overworld = world.getDimension("overworld")

let formMap = new Map<Entity, boolean>()

world.events.beforeChat.subscribe(e => {
    if (e.message.trim() == "#debug_stick") {
        e.cancel = true
        let player = e.sender
        let itemStack = new ItemStack(MinecraftItemTypes.stick, 1, 0)
        itemStack.setLore(["§r§bUse it on a block to set properties."])
        itemStack.nameTag = "§r§dDebug Stick§r"
        player.dimension.spawnItem(itemStack, player.location)
    }
})

world.events.tick.subscribe(async e => {
    for (let player of world.getPlayers()) {
        new PlayerInventoryComponentContainer
        let block = player.getBlockFromViewVector()
        if (!block) player.runCommand(`title @s actionbar §r`)
        let result = `§6§lid: §r§6${block.id}§r\n`

        block.permutation.getAllProperties().forEach(e => {
            let _e = e as BoolBlockProperty | IntBlockProperty | StringBlockProperty
            result += `§l§2${_e.name}: §r§2${_e.value}§r\n`
        })

        block.getTags().forEach(tag => {
            result += `§9#${tag}§r\n`
        })
        player.runCommand(`title @s actionbar ${result}`)
    }
})

world.events.beforeItemUseOn.subscribe(async e => {
    let { source: player, blockLocation, item } = e
    if (!(item.id == "minecraft:stick" && item.nameTag == "§r§5Debug Stick" && (item.getLore()[0] == "§r§bUse it on a block to set properties.")))
        if (item.getLore()[0] != "§r§bUse it on a block to set properties.")
            return;
    if (formMap.get(player)) return
    e.cancel = true
    let block = overworld.getBlock(blockLocation);
    if (!player.isSneaking) {
        let result = `§6§lid: §r§6${block.id}§r\n`

        block.permutation.getAllProperties().forEach(e => {
            let _e = e as BoolBlockProperty | IntBlockProperty | StringBlockProperty
            result += `§l§2${_e.name}: §r§2${_e.value}§r\n`
        })

        block.getTags().forEach(tag => {
            result += `§9#${tag}§r\n`
        })
        player.runCommand(`title @s actionbar ${result}`)
        return
    }
    let perm = block.permutation.clone()
    let p = perm.getAllProperties()
    if (p.length == 0) {
        player.runCommand(`title @s actionbar §cThe targeted block has no properties.`)
        return
    }
    const form = new ModalFormData()
        .title(block.id)
    let properties: IBlockProperty[] = []
    let type = MinecraftBlockTypes.get(block.id)
    for (let prop of p) {
        if (prop instanceof StringBlockProperty) {
            properties.push(prop)
            form.dropdown(prop.name, prop.validValues, prop.validValues.indexOf(prop.value))
        }
        if (prop instanceof BoolBlockProperty) {
            properties.push(prop)
            form.toggle(prop.name, prop.value)
        }
        if (prop instanceof IntBlockProperty) {
            properties.push(prop)
            form.dropdown(prop.name, prop.validValues.map(e => e.toString()), prop.validValues.indexOf(prop.value))
        }
        let pr = prop as StringBlockProperty
    }
    if (type.canBeWaterlogged) {
        form.toggle("isWaterLogged", block.isWaterlogged)
    }
    formMap.set(player, true)
    let res: {
        isCanceled: boolean,
        formValues: any[]
    } = await form.show(player)
    formMap.set(player, false)
    if (res.isCanceled) return
    res.formValues.forEach((e, i) => {
        let prop = properties[i]
        if (!prop && typeof e == "boolean") {
            block.isWaterlogged = e
        }
        if (prop instanceof StringBlockProperty) {
            let name = prop.name,
                value = prop.validValues[e]
            prop.value = value
        }
        if (prop instanceof BoolBlockProperty) {
            let name = prop.name,
                value = e
            prop.value = e
        }
        if (prop instanceof IntBlockProperty) {
            let name = prop.name,
                value = prop.validValues[e]
            prop.value = value
        }
    })
    block.setPermutation(perm)
    player.runCommand(`title @s actionbar §aProperties has been set`)
})
